import { StyleSheet } from '@emotion/sheet';
import { isBrowser } from '@emotion/utils';
import Stylis from '@emotion/stylis';

// https://github.com/thysultan/stylis.js/tree/master/plugins/rule-sheet
// inlined to avoid umd wrapper and peerDep warnings/installing stylis
// since we use stylis after closure compiler
var delimiter = '/*|*/';
var needle = delimiter + '}';

function toSheet(block) {
  if (block) {
    current.push(block + '}');
  }
}

var ruleSheet = function ruleSheet(context, content, selectors, parents, line, column, length, at, depth) {
  switch (context) {
    case -1:
      {
        current = [];
        break;
      }

    case 2:
      if (at === 0) return content + delimiter;
      break;
    // at-rule

    case 3:
      switch (at) {
        // @font-face, @page
        case 102:
        case 112:
          {
            current.push(selectors[0] + content);
            return '';
          }

        default:
          {
            return content + delimiter;
          }
      }

    case -2:
      {
        content.split(needle).forEach(toSheet);
        return current;
      }
  }
};

var current;

var createCache = function createCache(options) {
  if (options === undefined) options = {};
  var key = options.key || 'css';
  var stylisOptions;

  if (options.prefix !== undefined) {
    stylisOptions = {
      prefix: options.prefix
    };
  }

  var stylis = new Stylis(stylisOptions);
  stylis.use(options.stylisPlugins)(ruleSheet);

  if (process.env.NODE_ENV !== 'production') {
    // $FlowFixMe
    if (/[^a-z-]/.test(key)) {
      throw new Error("Emotion key must only contain lower case alphabetical characters and - but \"" + key + "\" was passed");
    }

    var sourceMapRegEx = /\/\*#\ssourceMappingURL=data:application\/json;\S+\s+\*\//;
    var currentSourceMap;
    stylis.use(function (context, content) {
      if (context === -1) {
        var result = sourceMapRegEx.exec(content);

        if (result) {
          currentSourceMap = result[0];
        }
      }

      if (context === -2) {
        if (currentSourceMap) {
          content.forEach(function (rule, i) {
            content[i] = rule + currentSourceMap;
          });
          currentSourceMap = '';
        }
      }
    });
  }

  var inserted = {};

  if (isBrowser) {
    var nodes = document.querySelectorAll( // $FlowFixMe
    "style[data-emotion-" + key + "]");
    Array.prototype.forEach.call(nodes, function (node) {
      // $FlowFixMe
      var attrib = node.getAttribute("data-emotion-" + key); // $FlowFixMe

      attrib.split(' ').forEach(function (id) {
        inserted[id] = true;
      }); // $FlowFixMe

      var parent = options.container || document.head;

      if (node.parentNode !== parent) {
        // $FlowFixMe
        parent.appendChild(node);
      }
    });
  }

  var context = {
    stylis: stylis,
    key: key,
    sheet: new StyleSheet({
      key: key,
      container: options.container,
      nonce: options.nonce
    }),
    inserted: inserted,
    registered: {},
    theme: {}
  };
  return context;
};

export default createCache;
//# sourceMappingURL=index.es.js.map
